__title__ = "socialname"
__version__ = "0.1.3"
__author__ = "Yusuke Hayashi"
__author_email__ = "yusuke8h@gmail.com"
__license__ = "MIT"
__description__ = "Find username across social network sites"
__url__ = "https://github.com/yhay81/socialname"



import concurrent.futures
import importlib
from typing import Any, Dict, List, Tuple, Optional

import psutil

from socialname.logics import BaseLogic, run_logic
from socialname.notify import QueryNotify, QueryNotifyPrint
from socialname.result import SocialNameResult
from socialname.sites import SiteInformation, create_sites_info


def get_max_workers(site_count: int, workers: int) -> int:
    # Limit number of workers to same as number of CPU.
    # Found on StackOverflow
    # https://stackoverflow.com/questions/1006289/how-to-find-out-the-number-of-cpus-using-python
    no_cpus = psutil.cpu_count(logical=True)
    if isinstance(no_cpus, int):
        # this will make sure number of worker will be same as logical CPU, so prevent overload/timeouts
        return min(site_count, no_cpus, workers)
    return min(site_count, workers)


def socialname(
    username: str,
    sites_info: Optional[Dict[str, SiteInformation]] = None,
    query_notify: Optional[QueryNotify] = None,
    *,
    workers: int = 20,
    **kwargs: Any,
) -> Dict[str, SocialNameResult]:
    if sites_info is None:
        sites_info = create_sites_info()
    if query_notify is None:
        query_notify = QueryNotifyPrint()

    query_notify.start(username)

    logic_usernames: List[Tuple[BaseLogic, str]] = []
    for site_name, site_info in sites_info.items():
        module = importlib.import_module(f"socialname.logics.{site_info.logic}")
        logic_usernames.append(
            (
                module.Logic(site_name, site_info, query_notify, **kwargs),  # type: ignore
                username,
            )
        )

    results: Dict[str, SocialNameResult] = {}
    max_workers = get_max_workers(len(sites_info), workers)
    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
        for result in executor.map(run_logic, logic_usernames):
            results[result.site_name] = result

    query_notify.finish()
    return results



from typing import Tuple

from socialname.result import SocialNameResult

from ._base import BaseLogic


def run_logic(logic_username: Tuple[BaseLogic, str]) -> SocialNameResult:
    logic, username = logic_username
    return logic.get_result(username)


__all__ = [
    "BaseLogic",
    "run_logic",
]



"""SocialName Module

This module contains the main logic to search for usernames at social
networks.
"""

from socialname.__version__ import (
    __author__,
    __author_email__,
    __license__,
    __title__,
    __version__,
)
from socialname.notify import QueryNotify, QueryNotifyPrint
from socialname.result import SocialNameResult, SocialNameStatus
from socialname.sites import SiteInformation, create_sites_info
from socialname.socialname import socialname

__all__ = [
    "__author__",
    "__author_email__",
    "__license__",
    "__title__",
    "__version__",
    "QueryNotify",
    "QueryNotifyPrint",
    "SiteInformation",
    "SocialNameResult",
    "SocialNameStatus",
    "create_sites_info",
    "socialname",
]



import re
import sys
import pathlib

import requests

from socialname.notify import QueryNotifyPrint
from socialname.sites import create_sites_info
from socialname.socialname import socialname
from socialname.output import write_csv, write_csv_singleoutput, open_browser
from socialname.__version__ import __version__

from .args import get_args, check_args


def check_newer_version() -> None:
    # Check for newer version of SocialName. If it exists, let the user know about it
    try:
        res = requests.get(
            "https://raw.githubusercontent.com/yhay81/socialname/master/socialname/__version__.py"
        )

        remote_version = str(re.findall('__version__ = "(.*)"', res.text)[0])
        local_version = __version__

        if remote_version != local_version:
            print(
                "Update Available!\n"
                + f"You are running version {local_version}. "
                + f"Version {remote_version} is available at https://git.io/sherlock"
            )

    except Exception as error:
        print(f"A problem occurred while checking for an update: {error}")


def execute() -> None:
    check_newer_version()
    args = get_args()
    check_args(args)

    # Create object with all information about sites we are aware of.
    data_file_path = (
        str(
            pathlib.Path(__file__)
            .parent.parent.joinpath("resources/data.json")
            .resolve()
        )
        if args.local
        else args.json_file
    )
    try:
        sites_info = create_sites_info(
            data_file_path=data_file_path, filter_list=args.site_list
        )
    except Exception as error:
        print(f"ERROR:  {error}")
        sys.exit(1)

    # Create notify object for query results.
    query_notify = QueryNotifyPrint(
        verbose=args.verbose,
        print_all=args.print_all,
        color=not args.no_color,
    )

    # Run report on all specified users.
    total_results = {
        username: socialname(
            username=username,
            sites_info=sites_info,
            query_notify=query_notify,
            tor=args.tor,
            unique_tor=args.unique_tor,
            proxy=args.proxy,
            timeout=args.timeout,
            workers=args.workers,
        )
        for username in args.username
    }

    if args.csv:
        if args.singleoutput:
            write_csv_singleoutput(total_results)
        else:
            write_csv(total_results)

    if args.browse:
        open_browser(total_results)


if __name__ == "__main__":
    execute()



from typing import Any, Callable, Optional, Tuple, Union, Dict
import dataclasses

import requests
import torrequest

from socialname.notify import QueryNotify
from socialname.result import SocialNameResult, SocialNameStatus
from socialname.sites import SiteInformation

from ._base import BaseLogic


@dataclasses.dataclass
class SimpleRequestContext:
    http_status: Union[str, int]
    elapsed_time: Optional[int]
    response_text: str
    error_context: Optional[str]
    expection_text: Optional[str]


def get_context(
    response: Optional[requests.Response],
    error_context: Optional[str],
    expection_text: Optional[str],
) -> SimpleRequestContext:
    elapsed_time: Optional[int] = None
    http_status: Union[str, int] = "?"
    response_text: str = ""
    if response is not None:
        # Get response time for response of our request.
        try:
            elapsed_time = response.elapsed.seconds
        except AttributeError:
            pass
        # Attempt to get request information
        try:
            http_status = response.status_code
        except Exception:  # nosec
            pass
        try:
            response_text = str(response.text.encode(response.encoding))
        except Exception:  # nosec
            pass
    return SimpleRequestContext(
        http_status=http_status,
        elapsed_time=elapsed_time,
        response_text=response_text,
        error_context=error_context,
        expection_text=expection_text,
    )


def get_response(
    url: str,
    request_method: Callable[..., requests.Response],
    allow_redirects: bool,
    headers: Dict[str, Any],
    proxy: Optional[str],
    timeout: Optional[int],
) -> Tuple[Optional[requests.Response], SimpleRequestContext]:
    response: Optional[requests.Response] = None
    error_context: Optional[str] = "General Unknown Error"
    expection_text = None
    try:
        response = request_method(
            url=url,
            headers={
                "User-Agent": (
                    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.12; rv:55.0) "
                    "Gecko/20100101 Firefox/55.0"
                ),
                # Override/append any extra headers required by a given site.
                **headers,
            },
            proxies={"http": proxy, "https": proxy} if proxy is not None else None,
            allow_redirects=allow_redirects,
            timeout=timeout,
        )

        if response is not None and response.status_code:
            # Status code exists in response object
            error_context = None
    except requests.exceptions.HTTPError as http_error:
        error_context = "HTTP Error"
        expection_text = str(http_error)
    except requests.exceptions.ProxyError as proxy_error:
        error_context = "Proxy Error"
        expection_text = str(proxy_error)
    except requests.exceptions.ConnectionError as connection_error:
        error_context = "Error Connecting"
        expection_text = str(connection_error)
    except requests.exceptions.Timeout as timeout_error:
        error_context = "Timeout Error"
        expection_text = str(timeout_error)
    except requests.exceptions.RequestException as error:
        error_context = "Unknown Error"
        expection_text = str(error)

    return response, get_context(response, error_context, expection_text)


class BaseSimpleRequestLogic(BaseLogic):
    underlying_request: Union[requests.Request, torrequest.TorRequest]
    underlying_session: requests.Session

    def __init__(
        self,
        site_name: str,
        site_info: SiteInformation,
        query_notify: QueryNotify,
        **kwargs: Any,
    ) -> None:
        super().__init__(site_name, site_info, query_notify, **kwargs)
        if kwargs.get("tor") is True or kwargs.get("unique_tor") is True:
            # Requests using Tor obfuscation
            self.underlying_request = torrequest.TorRequest()
            self.underlying_session = self.underlying_request.session
        else:
            # Normal requests
            self.underlying_request = requests.Request()
            self.underlying_session = requests.session()

    def get_result(self, username: str) -> SocialNameResult:
        site_name = self.site_info.site_name
        url_main = self.site_info.url_main
        url_user = self.site_info.url_user.format(username)
        url_probe = (
            self.site_info.url_probe.format(username)
            if self.site_info.url_probe is not None
            else None
        )

        # Don't make request if username is invalid for the site
        if self.is_illegal(username):
            # No need to do the check at the site: this user name is not allowed.
            return SocialNameResult(
                username=username,
                site_name=site_name,
                url_user=url_user,
                url_main=url_main,
                status=SocialNameStatus.ILLEGAL,
                elapsed_time=None,
                context={},
            )

        response, context = get_response(
            url=url_probe or url_user,
            request_method=self.request_method,
            allow_redirects=self.allow_redirects,
            headers=self.site_info.options.get("headers", {}),
            proxy=self.kwargs.get("proxy", None),
            timeout=self.kwargs.get("timeout", None),
        )

        if self.kwargs.get("unique_tor") is True and isinstance(
            self.underlying_request, torrequest.TorRequest
        ):
            self.underlying_request.reset_identity()

        if response is None or context.error_context is not None:
            status = SocialNameStatus.UNKNOWN
        else:
            status = self.get_status(response)

        result = SocialNameResult(
            username=username,
            site_name=site_name,
            url_user=url_user,
            url_main=url_main,
            status=status,
            elapsed_time=context.elapsed_time,
            context=dataclasses.asdict(context),
        )
        # Notify caller about results of query.
        self.query_notify.update(result)
        return result

    def get_status(self, response: requests.Response) -> SocialNameStatus:
        raise NotImplementedError

    @property
    def request_method(self) -> Callable[..., requests.Response]:
        raise NotImplementedError

    @property
    def allow_redirects(self) -> bool:
        raise NotImplementedError



from typing import Any
import re

from socialname.notify import QueryNotify
from socialname.result import SocialNameResult
from socialname.sites import SiteInformation

# Also need OPTIONS_VALIDATOR when implementing new logic
#
# import cerberus
# OPTIONS_VALIDATOR = cerberus.Validator({})
#


class BaseLogic:
    def __init__(
        self,
        site_name: str,
        site_info: SiteInformation,
        query_notify: QueryNotify,
        **kwargs: Any,
    ) -> None:
        self.site_name = site_name
        self.site_info = site_info
        self.query_notify = query_notify
        self.kwargs = kwargs

    def get_result(self, username: str) -> SocialNameResult:
        raise NotImplementedError

    def is_illegal(self, username: str) -> bool:
        regex_check = self.site_info.regex_check
        if regex_check is not None and re.search(regex_check, username) is None:
            return True
        return False



import json
from typing import Any, Dict
import importlib

import requests
import cerberus


VALIDATOR = cerberus.Validator(
    {
        "logic": {"type": "string", "required": True},
        "urlMain": {"type": "string", "required": True},
        "urlUser": {"type": "string", "required": True},
        "urlProbe": {"type": "string"},
        "regexCheck": {"type": "string"},
        "usernameClaimed": {"type": "string", "required": True},
        "usernameUnclaimed": {"type": "string", "required": True},
        "options": {"type": "dict", "allow_unknown": True},
    }
)


def validate(data: Dict[str, Any]) -> None:
    errors: Dict[str, str] = {}
    for key, value in data.items():
        is_valid = VALIDATOR.validate(value)
        if not is_valid:
            errors[key] = VALIDATOR.errors
            continue
        logic: str = value["logic"]
        module = importlib.import_module(f"socialname.logics.{logic}")
        options_validator: cerberus.Validator = module.OPTIONS_VALIDATOR  # type: ignore
        is_options_valid = options_validator.validate(value.get("options", {}))
        if not is_options_valid:
            errors[key] = options_validator.errors
    if len(errors) > 0:
        raise ValueError(f"Loaded data is not valid. {errors}")


def from_url(data_file_path: str) -> Dict[str, Any]:
    # Reference is to a URL.
    try:
        response = requests.get(url=data_file_path)
    except Exception as error:
        raise FileNotFoundError(
            f"Problem while attempting to access "
            f"data file URL '{data_file_path}':  {str(error)}"
        )
    if response.status_code == 200:
        try:
            result: Dict[str, Any] = response.json()
        except Exception as error:
            raise ValueError(
                f"Problem parsing json contents at "
                f"'{data_file_path}':  {str(error)}."
            )
        validate(result)
        return result
    raise FileNotFoundError(
        f"Bad response while accessing data file URL '{data_file_path}'."
    )


def from_file(data_file_path: str) -> Dict[str, Any]:
    # Reference is to a file.
    try:
        with open(data_file_path, "r", encoding="utf-8") as file:
            try:
                result: Dict[str, Any] = json.load(file)
            except Exception as error:
                raise ValueError(
                    f"Problem parsing json contents at "
                    f"'{data_file_path}':  {str(error)}."
                )
            validate(result)
            return result
    except FileNotFoundError:
        raise FileNotFoundError(
            f"Problem while attempting to access data file '{data_file_path}'."
        )



import csv
import os
import webbrowser
from typing import Dict, List, Tuple, Optional

from socialname.result import SocialNameStatus, SocialNameResult


def write_txt(
    total_results: Dict[str, Dict[str, SocialNameResult]],
    output: Optional[str] = None,
    folderoutput: Optional[str] = None,
) -> None:
    for username, results in total_results.items():
        if output is not None:
            result_file = output
        elif folderoutput is not None:
            # The usernames results should be stored in a targeted folder.
            # If the folder doesn't exist, create it first
            os.makedirs(folderoutput, exist_ok=True)
            result_file = os.path.join(folderoutput, f"{username}.txt")
        else:
            result_file = f"{username}.txt"

        with open(result_file, "w", encoding="utf-8") as file:
            exists_counter = 0
            for result in results.values():
                if result.status is SocialNameStatus.CLAIMED:
                    exists_counter += 1
                    file.write(result.url_user + "\n")
            file.write(f"Total Websites Username Detected On : {exists_counter}\n")


def write_csv_singleoutput(
    total_results: Dict[str, Dict[str, SocialNameResult]]
) -> None:
    csv_data: List[Tuple[str, str, str, str, str, str]] = [
        ("username", "site_name", "url_main", "url_user", "status", "response_time_s"),
    ]
    for username, results in total_results.items():
        for site_name, result in results.items():
            if result.status is SocialNameStatus.CLAIMED:
                csv_data.append(
                    (
                        username,
                        site_name,
                        result.url_main,
                        result.url_user,
                        str(result.status),
                        str(result.elapsed_time) or "",
                    )
                )
    with open("master.csv", "a", newline="", encoding="utf-8") as csv_file:
        writer = csv.writer(csv_file)
        writer.writerows(csv_data)


def write_csv(total_results: Dict[str, Dict[str, SocialNameResult]]) -> None:
    for username, results in total_results.items():
        csv_data: List[Tuple[str, str, str, str, str, str]] = [
            (
                "username",
                "site_name",
                "url_main",
                "url_user",
                "status",
                "response_time_s",
            ),
        ]
        for site_name, result in results.items():
            csv_data.append(
                (
                    username,
                    site_name,
                    result.url_main,
                    result.url_user,
                    str(result.status),
                    str(result.elapsed_time) or "",
                )
            )
        with open(username + ".csv", "w", newline="", encoding="utf-8") as csv_file:
            writer = csv.writer(csv_file)
            writer.writerows(csv_data)


def open_browser(total_results: Dict[str, Dict[str, SocialNameResult]]) -> None:
    # opening web browser after results are computed
    for results in total_results.values():
        for result in results.values():
            if result.status is SocialNameStatus.CLAIMED:
                webbrowser.open_new_tab(result.url_user)



from typing import Callable

import cerberus
import requests

from socialname.result import SocialNameStatus

from ._base_simple_request import BaseSimpleRequestLogic

OPTIONS_VALIDATOR = cerberus.Validator({})
OPTIONS_VALIDATOR.allow_unknown = True


class Logic(BaseSimpleRequestLogic):
    def get_status(self, response: requests.Response) -> SocialNameStatus:
        # Checks if the status code of the response is 2XX
        return (
            SocialNameStatus.CLAIMED
            if not response.status_code >= 300 or response.status_code < 200
            else SocialNameStatus.AVAILABLE
        )

    @property
    def request_method(self) -> Callable[..., requests.Response]:
        if self.site_info.options.get("requestHeadOnly", None) is True:
            # In most cases when we are detecting by status code,
            # it is not necessary to get the entire body:  we can
            # detect fine with just the HEAD response.
            return self.underlying_session.head
        # Either this detect method needs the content associated
        # with the GET response, or this specific website will
        # not respond properly unless we request the whole page.
        return self.underlying_session.get

    @property
    def allow_redirects(self) -> bool:
        return True



from typing import Callable

import cerberus
import requests

from socialname.result import SocialNameStatus

from ._base_simple_request import BaseSimpleRequestLogic

OPTIONS_VALIDATOR = cerberus.Validator(
    {
        "errorMessage": {
            "anyof": [
                {"type": "string"},
                {"type": "list", "schema": {"type": "string"}},
            ],
            "required": True,
        },
    }
)
OPTIONS_VALIDATOR.allow_unknown = True


class Logic(BaseSimpleRequestLogic):
    def get_status(self, response: requests.Response) -> SocialNameStatus:
        error_message = self.site_info.options.get("errorMessage", None)
        # error_flag True denotes no error found in the HTML
        # error_flag False denotes error found in the HTML
        error_flag = True
        # errors will hold the error message
        # it can be string or list
        # by insinstance method we can detect that
        # and handle the case for strings as normal procedure
        # and if its list we can iterate the errors
        if isinstance(error_message, str):
            # Checks if the error message is in the HTML
            # if error is present we will set flag to False
            if error_message in response.text:
                error_flag = False
        elif isinstance(error_message, list):
            # If it's list, it will iterate all the error message
            for error in error_message:
                if response is not None and error in response.text:
                    error_flag = False
                    break
        return SocialNameStatus.CLAIMED if error_flag else SocialNameStatus.AVAILABLE

    @property
    def request_method(self) -> Callable[..., requests.Response]:
        return self.underlying_session.get

    @property
    def allow_redirects(self) -> bool:
        return True



from typing import Union
import argparse
import platform
import sys

import requests
from socialname.__version__ import __version__, __title__


def timeout_check(value: Union[float, int, str]) -> float:
    """Check Timeout Argument.

    Checks timeout for validity.

    Keyword Arguments:
    value                  -- Time in seconds to wait before timing out request.

    Return Value:
    Floating point number representing the time (in seconds) that should be
    used for the timeout.

    NOTE:  Will raise an exception if the timeout in invalid.
    """

    try:
        timeout = float(value)
    except Exception:
        raise argparse.ArgumentTypeError(f"Timeout '{value}' must be a number.")
    if timeout <= 0:
        raise argparse.ArgumentTypeError(
            f"Timeout '{value}' must be greater than 0.0s."
        )
    return timeout


def get_args() -> argparse.Namespace:
    version_string = (
        f"%(prog)s {__version__}\n"
        + f"{requests.__title__}:  {requests.__version__}\n"
        + f"Python:  {platform.python_version()}"
    )

    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=f"{__title__} (Version {__version__})",
    )
    parser.add_argument(
        "--version",
        action="version",
        version=version_string,
        help="Display version information and dependencies.",
    )
    parser.add_argument(
        "--verbose",
        "-v",
        "-d",
        "--debug",
        action="store_true",
        dest="verbose",
        default=False,
        help="Display extra debugging information and metrics.",
    )
    parser.add_argument(
        "--folderoutput",
        "-fo",
        dest="folderoutput",
        help=(
            "If using multiple usernames, "
            "the output of the results will be saved to this folder."
        ),
    )
    parser.add_argument(
        "--output",
        "-o",
        dest="output",
        help=(
            "If using single username, "
            "the output of the result will be saved to this file."
        ),
    )
    parser.add_argument(
        "--tor",
        "-t",
        action="store_true",
        dest="tor",
        default=False,
        help=(
            "Make requests over Tor; increases runtime; "
            "requires Tor to be installed and in system path."
        ),
    )
    parser.add_argument(
        "--unique-tor",
        "-u",
        action="store_true",
        dest="unique_tor",
        default=False,
        help=(
            "Make requests over Tor with new Tor circuit after each request;"
            " increases runtime; requires Tor to be installed and in system path."
        ),
    )
    parser.add_argument(
        "--csv",
        action="store_true",
        dest="csv",
        default=False,
        help="Create Comma-Separated Values (CSV) File.",
    )
    parser.add_argument(
        "--site",
        action="append",
        metavar="SITE_NAME",
        dest="site_list",
        default=None,
        help=(
            "Limit analysis to just the listed sites. "
            "Add multiple options to specify more than one site."
        ),
    )
    parser.add_argument(
        "--proxy",
        "-p",
        metavar="PROXY_URL",
        action="store",
        dest="proxy",
        default=None,
        help="Make requests over a proxy. e.g. socks5://127.0.0.1:1080",
    )
    parser.add_argument(
        "--json",
        "-j",
        metavar="JSON_FILE",
        dest="json_file",
        default=None,
        help="Load data from a JSON file or an online, valid, JSON file.",
    )
    parser.add_argument(
        "--timeout",
        action="store",
        metavar="TIMEOUT",
        dest="timeout",
        type=timeout_check,
        default=None,
        help="Time (in seconds) to wait for response to requests. "
        "Default timeout is infinity. "
        "A longer timeout will be more likely to get results from slow sites. "
        "On the other hand, this may cause a long delay to gather all results.",
    )
    parser.add_argument(
        "--print-all",
        action="store_true",
        dest="print_all",
        help="Output sites where the username was not found.",
    )
    parser.add_argument(
        "--print-found",
        action="store_false",
        dest="print_all",
        default=False,
        help="Output sites where the username was found.",
    )
    parser.add_argument(
        "--no-color",
        action="store_true",
        dest="no_color",
        default=False,
        help="Don't color terminal output",
    )
    parser.add_argument(
        "username",
        nargs="+",
        metavar="USERNAMES",
        action="store",
        help="One or more usernames to check with social networks.",
    )
    parser.add_argument(
        "--browse",
        "-b",
        action="store_true",
        dest="browse",
        default=False,
        help="Browse to all results on default browser.",
    )

    parser.add_argument(
        "--local",
        "-l",
        action="store_true",
        default=True,
        help="Force the use of the local data.json file.",
    )

    parser.add_argument(
        "--singleoutput",
        "-s",
        action="store_true",
        default=False,
        help="Store all the outputs into 1 single file",
    )

    parser.add_argument(
        "--workers", "-w", type=int, default=20, help="Number of workers"
    )

    return parser.parse_args()


def check_args(args: argparse.Namespace) -> None:
    # Argument check
    # TODO(yhay81) regex check on args.proxy
    if args.tor and (args.proxy is not None):
        raise ValueError("Tor and Proxy cannot be set at the same time.")

    # Make prompts
    if args.proxy is not None:
        print("Using the proxy: " + args.proxy)

    if args.tor or args.unique_tor:
        print("Using Tor to make requests")
        print(
            (
                "Warning: some websites might refuse connecting over Tor, "
                "so note that using this option might increase connection errors."
            )
        )

    # Check if both output methods are entered as input.
    if args.output is not None and args.folderoutput is not None:
        print("You can only use one of the output methods.")
        sys.exit(1)

    # Check validity for single username output.
    if args.output is not None and len(args.username) != 1:
        print("You can only use --output with a single username")
        sys.exit(1)



"""SocialName Notify Module

This module defines the objects for notifying the caller about the
results of queries.
"""
import dataclasses
from typing import Optional, Union

from colorama import Fore, Style, init

from socialname.result import SocialNameResult, SocialNameStatus


class QueryNotify:
    """Query Notify Object.

    Base class that describes methods available to notify the results of
    a query.
    It is intended that other classes inherit from this base class and
    override the methods to implement specific functionality.
    """

    def __init__(self, result: Optional[SocialNameResult] = None) -> None:
        """Create Query Notify Object.

        Contains information about a specific method of notifying the results
        of a query.

        Keyword Arguments:
        self                 -- This object.
        result               -- Object of type QueryResult() containing
                                results for this query.

        Return Value:
        Nothing.
        """

        self.result = result

    def start(self, message: Optional[str] = None) -> None:
        """Notify Start.

        Notify method for start of query.  This method will be called before
        any queries are performed.  This method will typically be
        overridden by higher level classes that will inherit from it.

        Keyword Arguments:
        self                 -- This object.
        message              -- Object that is used to give context to start
                                of query.
                                Default is None.

        Return Value:
        Nothing.
        """

    def update(self, result: SocialNameResult) -> None:
        """Notify Update.

        Notify method for query result.  This method will typically be
        overridden by higher level classes that will inherit from it.

        Keyword Arguments:
        self                 -- This object.
        result               -- Object of type QueryResult() containing
                                results for this query.

        Return Value:
        Nothing.
        """

        self.result = result

    def finish(self, message: Optional[str] = None) -> None:
        """Notify Finish.

        Notify method for finish of query.  This method will be called after
        all queries have been performed.  This method will typically be
        overridden by higher level classes that will inherit from it.

        Keyword Arguments:
        self                 -- This object.
        message              -- Object that is used to give context to start
                                of query.
                                Default is None.

        Return Value:
        Nothing.
        """

    def __str__(self) -> str:
        """Convert Object To String.

        Keyword Arguments:
        self                   -- This object.

        Return Value:
        Nicely formatted string to get information about this object.
        """
        return str(self.result)


@dataclasses.dataclass
class PrintStyle:
    bright: Union[str, int] = ""
    reset_all: Union[str, int] = ""
    white: Union[str, int] = ""
    green: Union[str, int] = ""
    yellow: Union[str, int] = ""
    red: Union[str, int] = ""
    color: dataclasses.InitVar[bool] = False

    def __post_init__(self, color: bool) -> None:
        if color is True:
            self.bright = Style.BRIGHT
            self.reset_all = Style.RESET_ALL
            self.white = Fore.WHITE
            self.green = Fore.GREEN
            self.yellow = Fore.YELLOW
            self.red = Fore.RED


class QueryNotifyPrint(QueryNotify):
    """Query Notify Print Object.

    Query notify class that prints results.
    """

    def __init__(
        self,
        result: Optional[SocialNameResult] = None,
        verbose: bool = False,
        color: bool = True,
        print_all: bool = False,
    ) -> None:
        """Create Query Notify Print Object.

        Contains information about a specific method of notifying the results
        of a query.

        Keyword Arguments:
        self                 -- This object.
        result               -- Object of type QueryResult() containing
                                results for this query.
        verbose              -- Boolean indicating whether to give verbose output.
        print_all            -- Boolean indicating whether to only print all sites,
                                including not found.
        color                -- Boolean indicating whether to color terminal output

        Return Value:
        Nothing.
        """

        # Colorama module's initialization.
        init(autoreset=True)

        super().__init__(result)
        self.verbose = verbose
        self.print_all = print_all
        self.sty = PrintStyle(color=color)

    def start(self, message: Optional[str] = None) -> None:
        """Notify Start.

        Will print the title to the standard output.

        Keyword Arguments:
        self                 -- This object.
        message              -- String containing username that the series
                                of queries are about.

        Return Value:
        Nothing.
        """

        title = "Checking username"
        print(
            f"{self.sty.bright}{self.sty.green}[{self.sty.yellow}*{self.sty.green}] {title}"
            f"{self.sty.white} {message}{self.sty.green} on:"
        )

    def update(self, result: SocialNameResult) -> None:
        """Notify Update.

        Will print the query result to the standard output.

        Keyword Arguments:
        self                 -- This object.
        result               -- Object of type QueryResult() containing
                                results for this query.

        Return Value:
        Nothing.
        """
        self.result = result

        if self.verbose is False or result.elapsed_time is None:
            response_time_text = ""
        else:
            response_time_text = f" [{round(result.elapsed_time * 1000)} ms]"

        # Output to the terminal is desired.
        if result.status == SocialNameStatus.CLAIMED:
            print(
                (
                    f"{self.sty.bright}{self.sty.white}[{self.sty.green}+{self.sty.white}]{response_time_text} "
                    f"{self.sty.green}{result.site_name}: "
                    f"{self.sty.reset_all}{result.url_user}"
                )
            )

        elif result.status == SocialNameStatus.AVAILABLE:
            if self.print_all:
                print(
                    f"{self.sty.bright}{self.sty.white}[{self.sty.red}-{self.sty.white}]{response_time_text} "
                    f"{self.sty.green}{result.site_name}: "
                    f"{self.sty.yellow}Not Found!"
                )

        elif result.status == SocialNameStatus.UNKNOWN:
            if self.print_all:
                print(
                    f"{self.sty.bright}{self.sty.white}[{self.sty.red}-{self.sty.white}] "
                    f"{self.sty.green}{result.site_name}: "
                    f"{self.sty.red}{result.context}{self.sty.yellow} "
                )

        elif result.status == SocialNameStatus.ILLEGAL:
            if self.print_all:
                msg = "Illegal Username Format For This Site!"
                print(
                    f"{self.sty.bright}{self.sty.white}[{self.sty.red}-{self.sty.white}] "
                    f"{self.sty.green}{result.site_name} {self.sty.yellow}{msg}"
                )

        else:
            # It should be impossible to ever get here...
            raise ValueError(
                f"Unknown Query Status '{str(result.status)}' for "
                f"site '{result.site_name}'"
            )



#! /usr/bin/env python3

"""
SocialName: Find Usernames Across Social Networks Module

This module contains the main logic to search for usernames at social
networks.
"""

import sys


if __name__ == "__main__":
    # Checking if the user is using the correct version of Python
    # Reference:
    #  If Python version is 3.6.5
    #               major --^
    #               minor ----^
    #               micro ------^
    (major, minor, micro, _, __) = sys.version_info

    PYTHON_VERSION = f"{major}.{minor}.{micro}"

    if major != 3 or major == 3 and minor < 6:
        print(
            (
                f"SocialName requires Python 3.6+\nYou are using Python {PYTHON_VERSION}, "
                "which is not supported by SocialName"
            )
        )
        sys.exit(1)

    from socialname.cli import command

    command.execute()



#!/usr/bin/env python
import codecs
import sys
from typing import Dict

import setuptools

__version__: Dict[str, str] = {}
with codecs.open("socialname/__version__.py", "r") as version_file:
    exec(version_file.read(), __version__)  # noqa


install_requires = [
    "Cerberus>=1.3.2",
    "colorama>=0.4.4",
    "requests>=2.25.1",
    "torrequest>=0.1.0",
    "psutil>=5.8.0",
]

if sys.version_info[:2] == (3, 6):
    install_requires.append("dataclasses")

setuptools.setup(
    name=__version__["__title__"],
    version=__version__["__version__"],
    description=__version__["__description__"],
    long_description=codecs.open("README.md", "r").read(),
    long_description_content_type="text/markdown",
    keywords=["sns"],
    url=__version__["__url__"],
    author=__version__["__author__"],
    author_email=__version__["__author_email__"],
    license=__version__["__license__"],
    install_requires=install_requires,
    packages=setuptools.find_packages(exclude=["tests*"]),
    package_data={"": ["LICENSE"]},
    package_dir={"socialname": "socialname"},
    include_package_data=True,
    python_requires=">=3.6",
    zip_safe=False,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    project_urls={"Source": "https://github.com/yhay81/socialname"},
    extras_require={},
    entry_points={
        "console_scripts": [
            "socialname = socialname.cli.command:execute",
        ],
    },
)



"""SocialName Sites Information Module

This module supports storing information about web sites.
This is the raw data that will be used to search for usernames.
"""
import dataclasses
from typing import Any, Dict, Optional
import urllib.parse

from socialname import sites_load


@dataclasses.dataclass
class SiteInformation:
    """Site Information Object.

    Contains information about a specific web site.

    Keyword Arguments:
    name                 -- String which identifies site.
    url_main             -- String containing URL for home of site.
    url_user  -- String containing URL for Username format on site.
                            NOTE:  The string should contain the token "{}"
                            where the username should be substituted.
                            For example, a string of "https://somesite.com/users/{}"
                            indicates that the individual usernames would show up
                            under the "https://somesite.com/users/" area of the web site.
    username_claimed     -- String containing username which is known
                            to be claimed on web site.
    username_unclaimed   -- String containing username which is known
                            to be unclaimed on web site.
    information          -- Dictionary containing all known information
                            about web site.
                            NOTE:  Custom information about how to actually detect
                            the existence of the username will be included in this dictionary.
                            This information will be needed by the detection method,
                            but it is only recorded in this object for future use.
    """

    site_name: str
    logic: str = dataclasses.field(init=False, repr=False)
    url_main: str = dataclasses.field(init=False, repr=False)
    url_user: str = dataclasses.field(init=False, repr=False)
    url_probe: Optional[str] = dataclasses.field(init=False, repr=False)
    regex_check: Optional[str] = dataclasses.field(init=False, repr=False)
    username_claimed: str = dataclasses.field(init=False, repr=False)
    username_unclaimed: str = dataclasses.field(init=False, repr=False)
    options: Dict[str, Any] = dataclasses.field(init=False, repr=False)
    information: dataclasses.InitVar[Dict[str, Any]] = dataclasses.field(repr=False)

    def __post_init__(self, information: Dict[str, Any]) -> None:
        self.logic = information["logic"]
        self.url_main = information["urlMain"]
        self.url_user = information["urlUser"]
        self.url_probe = information.get("urlProbe", None)
        self.regex_check = information.get("regexCheck", None)
        self.username_claimed = information["usernameClaimed"]
        self.username_unclaimed = information["usernameUnclaimed"]
        self.options = information.get("options", {})

    def __str__(self) -> str:
        """Convert Object To String.

        Keyword Arguments:
        self                   -- This object.

        Return Value:
        Nicely formatted string to get information about this object.
        """

        return f"{self.site_name} ({self.url_main})"


def create_sites_info(
    data_file_path: Optional[str] = None, filter_list: Optional[str] = None
) -> Dict[str, SiteInformation]:

    sites_info: Dict[str, SiteInformation] = {}

    if data_file_path is None:
        # The default data file is the live data.json which is in the GitHub repo.
        # The reason why we are using this instead of the local one is so that
        # the user has the most up to date data. This prevents users from creating
        # issue about false positives which has already been fixed or having outdated data
        data_file_path = (
            "https://raw.githubusercontent.com"
            "/yhay81/socialname/master/socialname/resources/data.json"
        )
    # Ensure that specified data file has correct extension.
    if not data_file_path.lower().endswith(".json"):
        raise FileNotFoundError(
            f"Incorrect JSON file extension for data file '{data_file_path}'."
        )

    if urllib.parse.urlparse(data_file_path).scheme in ["http", "https"]:
        site_data = sites_load.from_url(data_file_path)
    else:
        site_data = sites_load.from_file(data_file_path)

    # Add all of site information from the json file to internal site list.
    if filter_list is None:
        for site_name, information in site_data.items():
            sites_info[site_name] = SiteInformation(
                site_name=site_name, information=information
            )
    else:
        for site_name in filter_list:
            if site_name in site_data:
                information = site_data[site_name]
                sites_info[site_name] = SiteInformation(
                    site_name=site_name, information=information
                )
            else:
                print(f"Error: Desired sites not found: {site_name}.")
    return sites_info



"""SocialName Result Module

This module defines various objects for recording the results of queries.
"""
import dataclasses
from enum import Enum
from typing import Optional, Dict, Any


class SocialNameStatus(Enum):
    """Query Status Enumeration.

    Describes status of query about a given username.
    """

    CLAIMED: str = "Claimed"  # Username Detected
    AVAILABLE: str = "Available"  # Username Not Detected
    UNKNOWN: str = "Unknown"  # Error Occurred While Trying To Detect Username
    ILLEGAL: str = "Illegal"  # Username Not Allowable For This Site

    def __str__(self) -> str:
        """Convert Object To String.

        Keyword Arguments:
        self                   -- This object.

        Return Value:
        Nicely formatted string to get information about this object.
        """
        return str(self.value)


@dataclasses.dataclass
class SocialNameResult:
    """Query Result Object.

    Describes result of query about a given username.
    Contains information about a specific method of detecting usernames on
    a given type of web sites.

    Keyword Arguments:
    username             -- String indicating username that query result
                            was about.
    site_name            -- String which identifies site.
    url_user        -- String containing URL for username on site.
                            NOTE:   The site may or may not exist:  this
                                    just indicates what the name would
                                    be, if it existed.
    status               -- Enumeration of type QueryStatus() indicating
                            the status of the query.
    query_time           -- Time (in seconds) required to perform query.
                            Default of None.
    context              -- String indicating any additional context
                            about the query.  For example, if there was
                            an error, this might indicate the type of
                            error that occurred.
                            Default of None.
    """

    username: str
    site_name: str
    url_main: str
    url_user: str
    status: SocialNameStatus
    elapsed_time: Optional[int]
    context: Dict[str, Any]

    def __str__(self) -> str:
        """Convert Object To String.

        Keyword Arguments:
        self                   -- This object.

        Return Value:
        Nicely formatted string to get information about this object.
        There is extra context information available about the results.
        Append it to the normal response text.
        """
        if not self.context:
            return str(self.status)
        return f"{str(self.status)} ({self.context})"



from typing import Callable

import requests

import cerberus
from socialname.result import SocialNameStatus

from ._base_simple_request import BaseSimpleRequestLogic

OPTIONS_VALIDATOR = cerberus.Validator(
    {
        "errorUrl": {"type": "string", "required": True},
    }
)
OPTIONS_VALIDATOR.allow_unknown = True


class Logic(BaseSimpleRequestLogic):
    def get_status(self, response: requests.Response) -> SocialNameStatus:
        # For this detection method, we have turned off the redirect.
        # So, there is no need to check the response URL: it will always
        # match the request.  Instead, we will ensure that the response
        # code indicates that the request was successful (i.e. no 404, or
        # forward to some odd redirect).
        return (
            SocialNameStatus.CLAIMED
            if 200 <= response.status_code < 300
            else SocialNameStatus.AVAILABLE
        )

    @property
    def request_method(self) -> Callable[..., requests.Response]:
        return self.underlying_session.get

    @property
    def allow_redirects(self) -> bool:
        return False
